/*
 * $RCSfile: RegistryKey.cc,v $
 * $Revision: 1.5 $
 *
 * Comments:
 *
 * (C) Copyright ParaSoft Corporation 1996.  All rights reserved.
 * THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF ParaSoft
 * The copyright notice above does not evidence any
 * actual or intended publication of such source code.
 *
 * $Author: mali $          $Locker:  $
 * $Date: 2008-11-18 12:02:54 $
 * $Log: not supported by cvs2svn $
 * Revision 1.4  2008/03/18 12:46:59  mali
 * error message functionality added
 *
 * Revision 1.3  2007/04/23 12:17:24  dario
 * try/catch added to destructor
 *
 * Revision 1.2  2006/01/25 12:21:15  dario
 * getValue modified (to be able to read REG_EXPAND_SZ values)
 *
 * Revision 1.1  2005/07/07 13:43:17  dario
 * added
 *
 */

#include "RegistryKey.h"
#include <sstream>

RegistryKey::RegistryKey(HKEY key, const string& sSubKey, int modeMask, Access access) :
    _bValid(false), _key(0)
{
    DWORD dwOptions = (access == READ) ? 
            (KEY_QUERY_VALUE | KEY_ENUMERATE_SUB_KEYS) :
            (KEY_ALL_ACCESS & ~(KEY_CREATE_LINK | WRITE_DAC | WRITE_OWNER));
            
    if (modeMask & OPEN) {
        LONG ret = RegOpenKeyEx(key, sSubKey.c_str(), 0, dwOptions, &_key);
        _bValid = (ret == ERROR_SUCCESS);
        if (!_bValid) {
            ostringstream oss;
            oss << "Cannot open registry key, return code: ";
            oss << ret;
            setErrorMessage(oss.str());
        }
    }
    if (!_bValid && (modeMask & CREATE)) {
        _bValid = createRegistryKey(key, sSubKey);
        if (_bValid) {
            setErrorMessage(""); // clean previous error message
        }
    }
}

RegistryKey::~RegistryKey(void)
{
	try {
	    if (isValid()) {
	        RegCloseKey(_key);
	    }
	} catch (...) { }
}

bool RegistryKey::keyNames(vector<string>& keyList)
{
    //char     ClassName[MAX_PATH] = "";
    //DWORD    dwcClassLen = MAX_PATH;   // Length of class string.
    DWORD    dwcSubKeys;               // Number of sub keys.
    DWORD    dwcMaxSubKey;             // Longest sub key size.
    DWORD    dwcMaxClass;              // Longest class string.
    DWORD    dwcValues;                // Number of values for this key.
    DWORD    dwcMaxValueName;          // Longest Value name.
    DWORD    dwcMaxValueData;          // Longest Value data.
    DWORD    dwcSecDesc;               // Security descriptor.
    FILETIME ftLastWriteTime;          // Last write time.
    
    long ret = RegQueryInfoKey(
        _key,
        NULL,
        NULL,              // Length of class string.
        0,                 // Reserved.
        &dwcSubKeys,       // Number of sub keys.
        &dwcMaxSubKey,     // Longest sub key size.
        &dwcMaxClass,      // Longest class string.
        &dwcValues,        // Number of values for this key.
        &dwcMaxValueName,  // Longest Value name.
        &dwcMaxValueData,  // Longest Value data.
        &dwcSecDesc,       // Security descriptor.
        &ftLastWriteTime); // Last write time.

    if (ret != ERROR_SUCCESS) {
        ostringstream oss;
        oss << "keyNames(), RegQueryInfoKey exit with code: ";
        oss << ret;
        setErrorMessage(oss.str());
        return false;
    }

    char* className  = new char[dwcMaxClass + 1];
    char* subkeyName = new char[dwcMaxSubKey + 1];
    
    for (int i = 0; i < dwcSubKeys; ++i) {
        DWORD classNameSz  = dwcMaxClass + 1;
        DWORD subkeyNameSz = dwcMaxSubKey + 1;
        
        ret = RegEnumKeyEx(
            _key,
            i,
            subkeyName, &subkeyNameSz,
            0,
            className, &classNameSz,
            &ftLastWriteTime
            );
            
        if (ret != ERROR_SUCCESS) {
            ostringstream oss;
            oss << "keyNames(), RegEnumKeyEx exit with code: ";
            oss << ret;
            setErrorMessage(oss.str());
            continue;
        }
        
        keyList.push_back(string(subkeyName));
    }
    
    delete [] className;
    delete [] subkeyName;
    
    return true;
}

bool RegistryKey::setValue(const string& sName, const string& sValue)
{
    LONG ret = RegSetValueEx(_key, sName.c_str(), 0, REG_SZ, (LPBYTE)sValue.c_str(), sValue.size() + 1);
    
    if (ret != ERROR_SUCCESS) {
        ostringstream oss;
        oss << "setValue(), RegSetValueEx exit with code: ";
        oss << ret;
        setErrorMessage(oss.str());
        return false;
    }
    return true;
}

bool RegistryKey::getValue(const string& sName, string& sValue)
{   
    DWORD dataType;
    DWORD dataSize;
    LONG ret = RegQueryValueEx(_key, sName.c_str(), 0, &dataType, 0, &dataSize);
    if ((ret != ERROR_SUCCESS) || ((dataType != REG_SZ) && (dataType != REG_EXPAND_SZ))) {
        ostringstream oss;
        oss << "getValue(), RegQueryValueEx exit with code: ";
        oss << ret;
        oss << ", dataType: ";
        oss << dataType;
        setErrorMessage(oss.str());
        return false;
    }
    
    char* buffer = new char[dataSize];
    ret = RegQueryValueEx(_key, sName.c_str(), 0, 0, (LPBYTE)buffer, &dataSize);
    if (ret != ERROR_SUCCESS) {
        ostringstream oss;
        oss << "getValue(), RegQueryValueEx exit with code: ";
        oss << ret;
        setErrorMessage(oss.str());
        return false;
    }
    sValue = buffer;
    delete [] buffer;
    return true;
}

bool RegistryKey::removeSubkey(const string& name)
{
    RegistryKey regKey(_key, name, OPEN, ALL);
    if (!regKey.isValid()) {
        return false;
    }
    vector<string> subkeyNames;
    regKey.keyNames(subkeyNames);
    vector<string>::const_iterator i = subkeyNames.begin();
    for ( ; i != subkeyNames.end(); ++i) {
        regKey.removeSubkey(*i);
    }
    LONG ret = RegDeleteKey(_key, name.c_str());
    if (ret != ERROR_SUCCESS) {
        ostringstream oss;
        oss << "removeSubkey(), Cannot delete registry key, return code: ";
        oss << ret;
        setErrorMessage(oss.str());
        return false;
    }
    return true;
}


bool RegistryKey::createRegistryKey(HKEY key, const string& sSubKey)
{
    DWORD disp;
    LONG ret = RegCreateKeyEx(key, sSubKey.c_str(), 0, "", REG_OPTION_NON_VOLATILE, 
            KEY_ALL_ACCESS & ~(KEY_CREATE_LINK | WRITE_DAC | WRITE_OWNER), 0, &_key, &disp);
    
    if (ret != ERROR_SUCCESS) {
        ostringstream oss;
        oss << "createRegistryKey(), Cannot create registry key, return code: ";
        oss << ret;
        setErrorMessage(oss.str());
        return false;
    }
    return true;
}
